@@ -1,16 +1,17 @@
 __all__ = [
-    'FAString'
+    'FAString',
+    'CatString',
 ]
 
 from functools import partial
-from typing import List, NamedTuple, Optional, Union
-import warnings
+from typing import List, NamedTuple, Optional, Union, TYPE_CHECKING
+if TYPE_CHECKING:
+    from .rt_dataset import Dataset
 
-try:
-    # This will be used to cache strlen for version of Python 3.7 and higher
-    from functools import cached_property
-except ImportError:
-    cached_property = property
+import warnings
+import inspect
+
+from functools import wraps
 
 import re
 import numpy as np
@@ -19,14 +20,16 @@ from numba.core.dispatcher import Dispatcher
 
 from .config import get_global_settings
 from .rt_fastarray import FastArray
-from .rt_numpy import empty_like, empty, where, ones, zeros
-from .rt_enum import TypeRegister
 
+from .rt_numpy import empty, where, ones, zeros, unique
+from .rt_enum import TypeRegister
+from .Utils.common import cached_property
 
 # Partially-specialize the numba.njit decorator to simplify its use in the FAString class below.
 _njit_serial = partial(nb.njit, parallel=False, cache=get_global_settings().enable_numba_cache, nogil=True)
 _njit_par = partial(nb.njit, parallel=True, cache=get_global_settings().enable_numba_cache, nogil=True)
 
+
 class FAStrDispatchPair(NamedTuple):
     """A pair of numba-based functions; one compiled for serial execution and the other for parallel execution."""
 
@@ -40,6 +43,11 @@ class FAStrDispatchPair(NamedTuple):
         return FAStrDispatchPair(serial=func_serial, parallel=func_par)
 
 
+
+def _warn_deprecated_naming(old_func, new_func):
+    warnings.warn(f"`{old_func}` is now deprecated and has been renamed to `{new_func}`", DeprecationWarning, stacklevel=2)
+
+
 # NOTE YOU MUST INSTALL tbb
 # conda install tbb
 # to confirm...
@@ -48,18 +56,18 @@ class FAStrDispatchPair(NamedTuple):
 # >>> 'tbb'
 #
 # NOTE this class was first written reshaping the array as 2d
-#----
-#if intype[1]=='S':
+# ----
+# if intype[1]=='S':
 #    return self.view(np.uint8).reshape((len(self), self.itemsize))
-#if intype[1]=='U':
+# if intype[1]=='U':
 #    return self.view(np.uint32).reshape((len(self), self.itemsize//4))
-#---- 
-#then pass to numba as 2d using shape[0] outer loop, shape[1] inner loop with [row,col] accessors
-#---
-#then flip back to string
-#if self.itemsize == 4:
+# ----
+# then pass to numba as 2d using shape[0] outer loop, shape[1] inner loop with [row,col] accessors
+# ---
+# then flip back to string
+# if self.itemsize == 4:
 #    result=self.ravel().view('U'+str(self.shape[1]))
-#else:
+# else:
 #    result=self.ravel().view('S'+str(self.shape[1]))
 #
 # Keeping it as 1d and remembering the itemsize is 10% faster on large arrays, and even faster on small arrays
@@ -70,53 +78,41 @@ class FAStrDispatchPair(NamedTuple):
 class FAString(FastArray):
     """
     String accessor class for `FastArray`.
-
-    Notes
-    -----
-    TODO: Consider making this class generic, so if we call ``.str`` on a `FastArray`, we'll get an ``FAString[FastArray]``,
-          but if we call ``.str`` on a `Categorical`, we'll get an ``FAString[Categorical]``. This might be useful when
-          annotating the return values of some methods on FAString.
-    TODO: Consider whether we should implement a derived ``CategoricalString`` class, an instance of which would be
-          returned when calling ``Categorical.str``; the ``CategoricalString`` class could override the implementations
-          of some methods to provide improved performance / semantics by e.g. operating on the category strings.
-          This refactoring would also remove the need for this class to check for and know the details of how to deal with
-          Categoricals -- that logic could be encapsulated entirely in the ``CategoricalString`` class.
     """
     _APPLY_PARALLEL_THRESHOLD = 10_000
 
-    def __new__(cls, arr, ikey=None, **kwargs):
+    def __new__(cls, arr):
         # if data comes in list like, convert to an array
         if not isinstance(arr, np.ndarray):
             if np.isscalar(arr):
-                arr=np.asanyarray([arr])
+                arr = np.asanyarray([arr])
             else:
-                arr=np.asanyarray(arr)
+                arr = np.asanyarray(arr)
 
-        intype=arr.dtype.char
-        if intype=='O':
+        intype = arr.dtype.char
+        if intype == 'O':
             # try to convert to string (might have come from pandas)
             # default to unicode (note: FastArray attempts 'S' first)
             arr = arr.astype('U')
-            intype=arr.dtype.char
+            intype = arr.dtype.char
 
         itemsize = np.int64(arr.itemsize)
 
-        if intype=='S':
+        if intype == 'S':
             # convert to two dim one byte array
             instance = arr.view(np.uint8)
-        elif intype=='U':
+        elif intype == 'U':
             # convert to two dim four byte array
             instance = arr.view(np.uint32)
-            itemsize = itemsize //4
+            itemsize = itemsize // 4
         else:
             raise TypeError(f"FAString can only be used on byte string and unicode not {arr.dtype!r}")
 
         instance = instance.view(cls)
 
         # remember the intype and itemsize
-        instance._intype=intype
-        instance._itemsize =itemsize
-        instance._ikey = ikey
+        instance._intype = intype
+        instance._itemsize = itemsize
         return instance
 
     # -----------------------------------------------------
@@ -150,24 +146,17 @@ class FAString(FastArray):
         # if data comes in list like, convert to an array
         if not isinstance(arr, np.ndarray):
             if not isinstance(arr, (list, tuple)):
-                arr=np.asanyarray([arr])
-            else: 
-                arr=np.asanyarray(arr)
+                arr = np.asanyarray([arr])
+            else:
+                arr = np.asanyarray(arr)
 
         if arr.dtype.char != self._intype:
             arr = arr.astype(self._intype)
 
         return arr
 
-    def _maybe_output_to_categorical(self, out):
-        if self._ikey is not None:
-            from .rt_categorical import Categorical
-            out = Categorical(self._ikey + 1, out, base_index=1)
-            out.category_make_unique(inplace=True)
-        return out
-
     # -----------------------------------------------------
-    def _apply_func(self, func, funcp, *args, dtype=None, input=None, filtered_fill_value=None):
+    def _apply_func(self, func, funcp, *args, dtype=None, input=None):
         # can optionally pass in dtype
         # check when to flip into parallel mode.  > 10,000 go to parallel routine
         # TODO: Can't just check len(self) for parallelism here; if operating on a string-mode Categorical
@@ -193,13 +182,6 @@ class FAString(FastArray):
         if dtype is None:
             dest = dest.view(dest._intype + str(dest._itemsize))
 
-        # check for categorical key re-expansion
-        if self._ikey is not None:
-            if dest.dtype.kind == 'S':
-                return self._maybe_output_to_categorical(dest)
-            else:
-                unfiltered = self._ikey >= 0
-                return where(unfiltered, dest[self._ikey], filtered_fill_value)
         return dest
 
     # -----------------------------------------------------
@@ -211,7 +193,7 @@ class FAString(FastArray):
 
         default signature must match
 
-        @nb.jit(nopython=True, cache=True)
+        @nb.njit(cache=get_global_settings().enable_numba_cache, nogil=True)
         def nb_upper(src, itemsize, dest):
 
         src: is uint array
@@ -226,9 +208,9 @@ class FAString(FastArray):
         Example:
         -------
         import numba as nb
-        @nb.jit(nopython=True, cache=True)
+        @nb.njit(cache=get_global_settings().enable_numba_cache, nogil=True)
         def nb_upper(src, itemsize, dest):
-            for i in range(len(src) / itemsize):
+            for i in nb.prange(len(src) / itemsize):
                 rowpos = i * itemsize
                 for j in range(itemsize):
                     c=src[rowpos+j]
@@ -251,10 +233,10 @@ class FAString(FastArray):
             rowpos = i * itemsize
             # loop over all chars in the string
             for j in range(itemsize):
-                c=src[rowpos+j]
+                c = src[rowpos + j]
                 if c >= 97 and c <= 122:
                     # convert to ASCII upper
-                    src[rowpos+j] = c-32
+                    src[rowpos + j] = c - 32
 
     # -----------------------------------------------------
     def _nb_upper(src, itemsize, dest):
@@ -263,12 +245,12 @@ class FAString(FastArray):
             rowpos = i * itemsize
             # loop over all chars in the string
             for j in range(itemsize):
-                c=src[rowpos+j]
+                c = src[rowpos + j]
                 if c >= 97 and c <= 122:
                     # convert to ASCII upper
-                    dest[rowpos+j] = c-32
+                    dest[rowpos + j] = c - 32
                 else:
-                    dest[rowpos+j] = c
+                    dest[rowpos + j] = c
 
     # -----------------------------------------------------
     def _nb_lower(src, itemsize, dest):
@@ -277,12 +259,12 @@ class FAString(FastArray):
             rowpos = i * itemsize
             # loop over all chars in the string
             for j in range(itemsize):
-                c=src[rowpos+j]
+                c = src[rowpos + j]
                 if c >= 65 and c <= 90:
                     # convert to ASCII lower
-                    dest[rowpos+j] = c+32
+                    dest[rowpos + j] = c + 32
                 else:
-                    dest[rowpos+j] = c
+                    dest[rowpos + j] = c
 
     # -----------------------------------------------------
     def _nb_removetrailing(src, itemsize, dest, removechar):
@@ -310,11 +292,11 @@ class FAString(FastArray):
         for i in nb.prange(len(src) // itemsize):
             rowpos = i * itemsize
             # find length of string
-            strlen=0
+            strlen = 0
             while (strlen < itemsize):
-                if src[rowpos + strlen] ==0: break
-                strlen +=1
-            end = rowpos + strlen -1
+                if src[rowpos + strlen] == 0: break
+                strlen += 1
+            end = rowpos + strlen - 1
             start = rowpos
             while (start < end):
                 temp = src[end]
@@ -329,18 +311,18 @@ class FAString(FastArray):
         for i in nb.prange(len(src) // itemsize):
             rowpos = i * itemsize
             # find length of string
-            strlen=0
+            strlen = 0
             while (strlen < itemsize):
-                if src[rowpos + strlen] ==0: break
-                strlen +=1
-            srcpos=0
-            while(strlen > 0):
+                if src[rowpos + strlen] == 0: break
+                strlen += 1
+            srcpos = 0
+            while (strlen > 0):
                 strlen -= 1
                 dest[rowpos + strlen] = src[rowpos + srcpos]
                 srcpos += 1
-            while(srcpos < itemsize):
+            while (srcpos < itemsize):
                 dest[rowpos + srcpos] = 0
-                srcpos +=1
+                srcpos += 1
 
     # -----------------------------------------------------
     def _nb_strlen(src, itemsize, dest):
@@ -348,7 +330,7 @@ class FAString(FastArray):
         for i in nb.prange(len(src) // itemsize):
             # loop over all chars in the string
             rowpos = i * itemsize
-            strlen= 0
+            strlen = 0
             # loop over all chars in the string
             for j in range(itemsize):
                 if src[rowpos + j] == 0:
@@ -358,18 +340,18 @@ class FAString(FastArray):
             dest[i] = strlen
 
     # -----------------------------------------------------
-    def _nb_strpbrk(src, itemsize, dest, str2):
+    def _nb_index_any_of(src, itemsize, dest, str2):
         str2len = len(str2)
         # loop over all rows
         for i in nb.prange(len(src) // itemsize):
             # loop over all chars in the string
             rowpos = i * itemsize
-            found =0
+            found = 0
             # loop over all chars in the string
             for j in range(itemsize):
-                c= src[rowpos + j]
+                c = src[rowpos + j]
                 for k in range(str2len):
-                    if c==str2[k]:
+                    if c == str2[k]:
                         # store location of match
                         dest[i] = j
                         found = 1
@@ -380,7 +362,7 @@ class FAString(FastArray):
                 dest[i] = -1
 
     # -----------------------------------------------------
-    def _nb_strstr(src, itemsize, dest, str2):
+    def _nb_index(src, itemsize, dest, str2):
         str2len = len(str2)
         # loop over all rows
         for i in nb.prange(len(src) // itemsize):
@@ -414,7 +396,7 @@ class FAString(FastArray):
                     if src[rowpos + j + k] != str2[k]:
                         break
                     k += 1
-                if k==str2len:
+                if k == str2len:
                     # indicate we have a match
                     dest[i] = True
                     break
@@ -430,25 +412,25 @@ class FAString(FastArray):
 
             # loop over all chars in the string
             # check if enough space left
-            if itemsize >= str2len: 
-                k =itemsize
-                while ((k > 0) and (src[rowpos + k -1]==0)):
+            if itemsize >= str2len:
+                k = itemsize
+                while ((k > 0) and (src[rowpos + k - 1] == 0)):
                     k -= 1
 
                 # check if still enough space left
-                if k >= str2len: 
+                if k >= str2len:
 
-                    k2=str2len
+                    k2 = str2len
                     # check if only the end matches
                     while (k2 > 0):
-                        if src[rowpos + k -1] != str2[k2-1]:
+                        if src[rowpos + k - 1] != str2[k2 - 1]:
                             break
                         k -= 1
-                        k2 -=1
-                    if k2==0:
+                        k2 -= 1
+                    if k2 == 0:
                         # indicate we have a match
                         dest[i] = True
-                    
+
     # -----------------------------------------------------
     def _nb_startswith(src, itemsize, dest, str2):
         str2len = len(str2)
@@ -459,14 +441,14 @@ class FAString(FastArray):
             dest[i] = False
             # loop over all chars in the string
             # check if enough space left
-            if itemsize >= str2len: 
-                k =0
+            if itemsize >= str2len:
+                k = 0
                 # check if only the beginning matches
                 while (k < str2len):
                     if src[rowpos + k] != str2[k]:
                         break
                     k += 1
-                if k==str2len:
+                if k == str2len:
                     # indicate we have a match
                     dest[i] = True
 
@@ -481,7 +463,7 @@ class FAString(FastArray):
         --------
         >>> FAString(['this','that','test']).upper
         FastArray(['THIS','THAT','TEST'], dtype='<U4')
-        
+
         '''
         return self._apply_func(self.nb_upper, self.nb_upper_par)
 
@@ -561,7 +543,7 @@ class FAString(FastArray):
         return self._apply_func(self.nb_removetrailing, self.nb_removetrailing_par, remove)
 
     # -----------------------------------------------------
-    @cached_property     # only cached for Python 3.7 or higher
+    @cached_property  # only cached for Python 3.7 or higher
     def strlen(self):
         '''
         return the string length of every string (bytes or unicode)
@@ -571,11 +553,10 @@ class FAString(FastArray):
         >>> FAString(['this  ','that ','test']).strlen
         FastArray([6, 5, 4])
         '''
-        return self._apply_func(self.nb_strlen, self.nb_strlen_par, dtype=np.int32,
-                                filtered_fill_value=np.iinfo(np.int32).min)
+        return self._apply_func(self.nb_strlen, self.nb_strlen_par, dtype=np.int32)
 
     # -----------------------------------------------------
-    def strpbrk(self, str2):
+    def index_any_of(self, str2):
         '''
         return the first index location any of the characters that are part of str2,
         or -1 if none of the characters match
@@ -583,23 +564,29 @@ class FAString(FastArray):
         Parameters
         ----------
         str2 - a string with one or more characters to search for
-        
+
         Examples
         --------
-        >>> FAString(['this  ','that ','test']).strpbrk('ia')
+        >>> FAString(['this  ','that ','test']).index_any_of('ia')
         FastArray([2, 2, -1])
         '''
         if not isinstance(str2, FAString):
+            if str2 == '':
+                return zeros(self.n_elements, dtype=np.int32)
+
             str2 = self.possibly_convert_tostr(str2)
             if len(str2) != 1:
                 return TypeError(f"A single string must be passed for str2 not {str2!r}")
             str2 = FAString(str2)
-       
-        return self._apply_func(self.nb_strpbrk, self.nb_strpbrk_par, str2, dtype=np.int32,
-                                filtered_fill_value=np.iinfo(np.int32).min)
+
+        return self._apply_func(self.nb_index_any_of, self.nb_index_any_of_par, str2, dtype=np.int32)
+
+    def strpbrk(self, str2):
+        _warn_deprecated_naming('strpbrk', 'index_any_of')
+        return self.index_any_of(str2)
 
     # -----------------------------------------------------
-    def strstr(self, str2):
+    def index(self, str2):
         '''
         return the first index location of the entire substring specified in str2,
         or -1 if the substring does not exist
@@ -607,10 +594,10 @@ class FAString(FastArray):
         Parameters
         ----------
         str2 - a string with one or more characters to search for
-        
+
         Examples
         --------
-        >>> FAString(['this  ','that ','test']).strstr('at')
+        >>> FAString(['this  ','that ','test']).index('at')
         FastArray([-1, 2, -1])
         '''
         if not isinstance(str2, FAString):
@@ -622,8 +609,11 @@ class FAString(FastArray):
                 return TypeError(f"A single string must be passed for str2 not {str2!r}")
             str2 = FAString(str2)
 
-        return self._apply_func(self.nb_strstr, self.nb_strstr_par, str2, dtype=np.int32,
-                                filtered_fill_value=np.iinfo(np.int32).min)
+        return self._apply_func(self.nb_index, self.nb_index_par, str2, dtype=np.int32)
+
+    def strstr(self, str2):
+        _warn_deprecated_naming('strstr', 'index')
+        return self.index(str2)
 
     # -----------------------------------------------------
     def contains(self, str2):
@@ -635,7 +625,7 @@ class FAString(FastArray):
         Parameters
         ----------
         str2 - a string with one or more characters to search for
-        
+
         Examples
         --------
         >>> FAString(['this  ','that ','test']).contains('at')
@@ -650,14 +640,10 @@ class FAString(FastArray):
                 return TypeError(f"A single string must be passed for str2 not {str2!r}")
             str2 = FAString(str2)
 
-        return self._apply_func(self.nb_contains, self.nb_contains_par, str2, dtype=np.bool,
-                                filtered_fill_value=False)
+        return self._apply_func(self.nb_contains, self.nb_contains_par, str2, dtype=bool)
 
     def strstrb(self, str2):
-        """
-        Deprecated. Please see .contains.
-        """
-        warnings.warn("strstrb is now deprecated and has been renamed to `contains`", DeprecationWarning)
+        _warn_deprecated_naming('strstrb', 'contains')
         return self.contains(str2)
 
     # -----------------------------------------------------
@@ -669,7 +655,7 @@ class FAString(FastArray):
         Parameters
         ----------
         str2 - a string with one or more characters to search for
-        
+
         Examples
         --------
         >>> FAString(['this  ','that ','test']).startswith('thi')
@@ -683,9 +669,8 @@ class FAString(FastArray):
             if len(str2) != 1:
                 return TypeError(f"A single string must be passed for str2 not {str2!r}")
             str2 = FAString(str2)
-       
-        return self._apply_func(self.nb_startswith, self.nb_startswith_par, str2, dtype=np.bool,
-                                filtered_fill_value=False)
+
+        return self._apply_func(self.nb_startswith, self.nb_startswith_par, str2, dtype=bool)
 
     # -----------------------------------------------------
     def endswith(self, str2):
@@ -696,7 +681,7 @@ class FAString(FastArray):
         Parameters
         ----------
         str2 - a string with one or more characters to search for
-        
+
         Examples
         --------
         >>> FAString(['abab','ababa','abababb']).endswith('ab')
@@ -711,8 +696,7 @@ class FAString(FastArray):
                 return TypeError(f"A single string must be passed for str2 not {str2!r}")
             str2 = FAString(str2)
 
-        return self._apply_func(self.nb_endswith, self.nb_endswith_par, str2, dtype=np.bool,
-                                filtered_fill_value=False)
+        return self._apply_func(self.nb_endswith, self.nb_endswith_par, str2, dtype=bool)
 
     def regex_match(self, regex):
         '''
@@ -734,16 +718,111 @@ class FAString(FastArray):
         regex = re.compile(regex)
         vmatch = np.vectorize(lambda x: bool(regex.search(x)))
         bools = vmatch(self.backtostring)
-        if self._ikey is not None:
-            bools = bools[self._ikey] & (self._ikey >= 0)
+
         return bools
 
+    def extract(self, regex: str, expand: Optional[bool] = None,
+                fillna: str = '', names=None, apply_unique: bool = True
+                ) -> Union[FastArray, "Dataset"]:
+        '''
+        Extract one or more pattern groups into a Dataset or FastArray.
+        For one capture group the default is to return a FastArray
+        but this can be overridden by passing expand=True or by providing names.
+        Column names can be specified within the regex using (?P<name>) in the search group(s)
+        or by passing the names argument which may be more convenient.
+
+        Parameters
+        ----------
+        regex: str
+            Contains the patterns to search for
+        expand: bool
+            set to True to return a Dataset for a single capture group.
+        fillna: str
+            Used for rows where no regex does not match
+        names: List[str]
+            Optional list of strings provides keys for resultant dataset
+        apply_unique: bool
+            When True we apply the regex to the unique values and then expand using the reverse index.
+            This is optimal for repetitive data and benign for unique or close highly non-repetitive data
+
+        Examples
+        --------
+        >>> osi = rt.FastArray(['SPX UO 12/15/23 C5700', 'SPXW UO 09/17/21 C3650'])
+        >>> osi.str.extract('\w+')
+        FastArray([b'SPX', b'SPXW'], dtype='|S4')
+
+        >>> osi.str.extract('(?P<root>\w+)')
+        #   root
+        -   ----
+        0   SPX
+        1   SPXW
+
+        >>> osi.str.extract('(\w+).* (\d{2}/\d{2}/\d{2})', names=['root', 'expiration'])
+        #   root   expiration
+        -   ----   ----------
+        0   SPX    12/15/23
+        1   SPXW   09/17/21
+
+        >>> osi.str.extract('\w+W', expand=True)
+        #   group_0
+        -   -------
+        0
+        1   SPXW
+        '''
+        kwargs = dict(expand=expand, fillna=fillna, names=names, apply_unique=apply_unique)
+        if apply_unique:
+            kwargs['apply_unique'] = False
+            unique_values, index = unique(self.backtostring, return_inverse=True)
+            result = unique_values.str.extract(regex, **kwargs)
+            return result[index] if isinstance(result, FastArray) else result[index, :]
+
+        if not isinstance(regex, bytes):
+            regex = bytes(regex, 'utf-8')
+        compiled = re.compile(regex)
+
+        ngroups = compiled.groups
+        if ngroups == 0:
+            # convenience where we treat the entire pattern as a capture group
+            return self.extract(f'({regex.decode()})', **kwargs)
+
+        # expand defaults to False if we have one capture group and do not specify names
+        if expand is None:
+            expand = ngroups > 1 or names is not None
+        elif not expand and ngroups > 1:
+            raise ValueError("expand cannot be False with multiple capture groups")
+
+        if names is None:
+            names = [f'group_{i}' for i in range(ngroups)]
+            for name, index in compiled.groupindex.items():
+                names[index - 1] = name
+        elif len(names) != ngroups:
+            raise ValueError(f"Number of names, {len(names)}, does not match number of groups, {ngroups}")
+
+        strings = self.backtostring
+        # we define a list containing an empty array for each group
+        # use Python lists as we do not know how many chars will be in the resultant arrays.
+        # Performance in comparable to unsing pre-allocated numpy arrays with ~600K unique elements
+        out_arrs = [[fillna] * len(strings) for _ in range(ngroups)]
+
+        for i, s in enumerate(strings):
+            result = compiled.search(s)
+            if result is not None:
+                result = result.groups()
+                for s, arr in zip(result, out_arrs):
+                    arr[i] = s
+
+        if expand:
+            out = TypeRegister.Dataset(dict(zip(names, out_arrs)))
+        else:
+            out = FastArray(out_arrs[0])
+        return out
+
     def _nb_substr(src, out, itemsize, start, stop, strlen):
         n_elements = len(out)
         max_chars = 0
         for elem in nb.prange(n_elements):
             elem_len = strlen[elem]
-            i, j = start, stop
+            i, j = start[elem], stop[elem]
             if i < 0:
                 i += elem_len
             if j < 0:
@@ -756,18 +835,8 @@ class FAString(FastArray):
                 max_chars = max(max_chars, out_pos + 1)
         return out[:, :max_chars]
 
-    @cached_property
-    def _cat_strlen(self):
-        """
-        Same as strlen except for Categoricals it is aligned with the categories
-        as opposed to the full array. Used for substring methods.
-        """
-        if self._ikey is None:
-            return self.strlen
-        else:
-            return FAString(self.backtostring).strlen
-
-    def substr(self, start: Union[int, np.ndarray], stop: Optional[Union[int, np.ndarray]] = None):
+    def _substr(self, start: Union[int, np.ndarray], stop: Optional[Union[int, np.ndarray]] = None
+               ) -> FastArray:
         """
         Take a substring of each element using slice args.
         """
@@ -775,28 +844,67 @@ class FAString(FastArray):
             # emulate behaviour of slice
             start, stop = 0, start
 
-        strlen = self._cat_strlen
+        def _bound_to_array(value, name):
+            value = TypeRegister.FastArray(value)
+            if len(value) == 1:
+                value = np.full(self.n_elements, value[0])
+            if value.shape != (self.n_elements,):
+                raise ValueError(
+                    f"{name} must be an integer or an array of length equal to the number of elements"
+                    f" in self. Expected {(self.n_elements,)}, got {value.shape}")
+            return value
 
-        if start < 0:
-            if stop < 0:
-                n_chars = stop - start
-            else:
-                n_chars = stop  # we can't tell what the max length at this point
-        elif stop < 0:
-            pos_stop = self._itemsize - stop
-            n_chars = pos_stop - start
+        start = _bound_to_array(start, 'start')
+        stop = _bound_to_array(stop, 'stop')
+
+        strlen = self.strlen
+
+        out = zeros((self.n_elements, self._itemsize), self.dtype)
+
+        if self.n_elements >= self._APPLY_PARALLEL_THRESHOLD:
+            substr = self.nb_substr_par
         else:
-            n_chars = stop - start
+            substr = self.nb_substr
+
+        out = substr(out, self._itemsize, start, stop, strlen)
 
-        out = zeros((self.n_elements, n_chars), self.dtype)
-        out = self._nb_substr(out, self._itemsize, start, stop, strlen)
         n_chars = out.shape[1]
-        if n_chars == 0:    # empty sub strings everywhere
+        if n_chars == 0:  # empty sub strings everywhere
             out = zeros(self.n_elements, self.dtype).view(f'{self._intype}1')
         else:
             out = out.ravel().view(f'<{self._intype}{n_chars}')
-        out = self._maybe_output_to_categorical(out)
-        return out
+        return FastArray(out)
+
+    @property
+    def substr(self):
+        return _SubStrAccessor(self)
+
+    def substr_char_stop(self, stop: str, inclusive: bool = False) -> FastArray:
+        """
+        Take a substring of each element using characters as bounds.
+
+        Parameters
+        ----------
+        stop:
+            A string used to determine the start of the sub-string.
+            Excluded from the result by default.
+            We go to the end of the string where stop is not in found in the corresponding element
+        inclusive: bool
+            If True, include the stopping string in the result
+
+        Examples
+        --------
+        >>> s = FastArray(['ABC', 'A_B', 'AB_C', 'AB_C_DD'])
+        >>> s.str.substr_char_stop('_')
+        FastArray([b'ABC', b'A', b'AB', b'AB'], dtype='|S2')
+        >>> s.str.substr_char_stop('_', inclusive=True)
+        FastArray([b'ABC', b'A_', b'AB_', b'AB_'], dtype='|S2')
+        """
+        int_stop = self.index(stop)
+        int_stop[int_stop == -1] = self._itemsize  # return full string if stop not found
+        if inclusive:
+            int_stop += 1
+        return self.substr(int_stop)
 
     def _nb_char(src, position, itemsize, strlen, out):
         broken_at = len(position)
@@ -808,7 +916,7 @@ class FAString(FastArray):
             if pos >= itemsize or pos < 0:
                 # Parallel reduction on this index.
                 # Otherwise, returning here prevents the function from being parallelized.
-                broken_at = np.minimum(broken_at, i)    # this triggers error below (in `char()`).
+                broken_at = np.minimum(broken_at, i)  # this triggers error below (in `char()`).
 
                 # TODO: Set out[i] to some invalid value?
                 out[i] = 0
@@ -817,7 +925,7 @@ class FAString(FastArray):
 
         return broken_at if broken_at < len(position) else -1
 
-    def char(self, position: Union[int, List[int], np.ndarray]):
+    def char(self, position: Union[int, List[int], np.ndarray]) -> FastArray:
         """
         Take a single character from each element.
 
@@ -843,14 +951,12 @@ class FAString(FastArray):
             raise ValueError("position must be a scalar or a vector of the same length as self")
 
         out = zeros(self.n_elements, self.dtype)
-        strlen = self._cat_strlen
-        broken_at = self._nb_char(position, self._itemsize, strlen, out)
+        broken_at = self._nb_char(position, self._itemsize, self.strlen, out)
         if broken_at >= 0:
             raise ValueError(f"Position {position[broken_at]} out of bounds "
                              f"for string of length {self._itemsize}")
         out = out.view(f'{self._intype}1')
-        return self._maybe_output_to_categorical(out)
-
+        return FastArray(out)
 
     # Use the specialized decorators to create both a serial and parallel version of each
     # numba function (so we only need one definition of each), then add it to FAString.
@@ -877,11 +983,11 @@ class FAString(FastArray):
     nb_strlen = _njit_serial(_nb_strlen)
     nb_strlen_par = _njit_par(_nb_strlen)
 
-    nb_strpbrk = _njit_serial(_nb_strpbrk)
-    nb_strpbrk_par = _njit_par(_nb_strpbrk)
+    nb_index_any_of = _njit_serial(_nb_index_any_of)
+    nb_index_any_of_par = _njit_par(_nb_index_any_of)
 
-    nb_strstr = _njit_serial(_nb_strstr)
-    nb_strstr_par = _njit_par(_nb_strstr)
+    nb_index = _njit_serial(_nb_index)
+    nb_index_par = _njit_par(_nb_index)
 
     nb_contains = _njit_serial(_nb_contains)
     nb_contains_par = _njit_par(_nb_contains)
@@ -899,5 +1005,128 @@ class FAString(FastArray):
     nb_char_par = _njit_par(_nb_char)
 
 
+def _populate_wrappers(cls):
+    """
+    Decorator for the CatString class which populates the string methods and sets the defaults for
+    filtered value fills.
+    """
+    functions = dict(inspect.getmembers(FAString, inspect.isfunction))
+    properties = dict(inspect.getmembers(FAString, lambda o: isinstance(o, (cached_property, property))))
+    for name in [
+        'upper',
+        'lower',
+        'reverse',
+        'removetrailing',
+        'strlen',
+        'index_any_of',
+        'index',
+        'contains',
+        'startswith',
+        'endswith',
+        'regex_match',
+        '_substr',
+        'char',
+
+        # Deprecated methods.
+        'strstr',
+        'strstrb',
+        'strpbrk'
+    ]:
+        if name in functions:
+            wrapper = cls._build_method(functions[name])
+        elif name in properties:
+            wrapper = cls._build_property(name)
+        else:
+            raise RuntimeError(f'{name} is not defined on FAString as a function or property')
+        setattr(cls, name, wrapper)
+    return cls
+
+
+class _SubStrAccessor:
+    """
+    Class for providing slicing on string arrays via FAString.substr
+    """
+    def __init__(self, fastring):
+        self.fastring = fastring
+
+    def __getitem__(self, y):
+        if isinstance(y, slice):
+            start = 0 if y.start is None else y.start
+            return self.fastring._substr(start, y.stop)
+        else:
+            return self.fastring.char(y)
+
+    def __call__(self, start, stop=None):
+        return self.fastring._substr(start, stop)
+
+
+@_populate_wrappers
+class CatString:
+    """
+    Provides access to FAString methods for Categoricals.
+    All string methods are wrappers of the FAString equivalent with
+    categorical re-expansion and option for how to fill filtered elements.
+    """
+
+    def __init__(self, cat):
+        from .rt_categorical import CategoryMode
+        self.cat = cat
+        if cat.category_mode == CategoryMode.StringArray:
+            string_list = cat.category_array
+            self.fastring = FAString(string_list)
+        else:
+            raise ValueError(f"Could not use .str in {CategoryMode(cat.category_mode).name}.")
+
+    @cached_property
+    def _isfiltered(self):
+        return self.cat.isfiltered()
+
+    def _convert_fastring_output(self, out):
+        if out.dtype.kind in 'SU':
+            out = TypeRegister.Categorical._from_maybe_non_unique_labels(self.cat._fa, out, base_index=self.cat.base_index)
+            return out
+        else:
+            return where(self._isfiltered, out.inv, out[self.cat.ikey - 1])
+
+    @classmethod
+    def _build_method(cls, method):
+        """
+        General purpose factory for FAString function wrappers.
+        """
+
+        @wraps(method)
+        def wrapper(self, *args, **kwargs):
+            out = method(self.fastring, *args, **kwargs)
+            return self._convert_fastring_output(out)
+
+        return wrapper
+
+    @classmethod
+    def _build_property(cls, name):
+        """
+        General purpose factory for FAString property wrappers.
+        """
+
+        def wrapper(self):
+            return self._convert_fastring_output(getattr(self.fastring, name))
+
+        return property(wrapper)
+
+    def extract(self, regex: str, expand: Optional[bool] = None,
+                fillna: str = '', names=None):
+        out = self.fastring.extract(regex, expand=expand, fillna=fillna, names=names, apply_unique=False)
+        if isinstance(out, TypeRegister.Dataset):
+            return TypeRegister.Dataset({key: self._convert_fastring_output(col) for key, col in out.items()})
+        else:
+            return self._convert_fastring_output(out)
+
+    extract.__doc__ = FAString.__doc__  # might be misleading since we drop the apply_unique argument
+
+    @property
+    def substr(self):
+        return _SubStrAccessor(self)
+
+
 # keep as last line
-TypeRegister.FAString=FAString
+TypeRegister.FAString = FAString
+TypeRegister.CatString = CatString
